# coding: utf-8

"""
    Agilicus API

    Agilicus API endpoints  # noqa: E501

    The version of the OpenAPI document: 2020.03.10
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest

import agilicus_api
from agilicus_api.api.issuers_api import IssuersApi  # noqa: E501
from agilicus_api.rest import ApiException


class TestIssuersApi(unittest.TestCase):
    """IssuersApi unit test stubs"""

    def setUp(self):
        self.api = agilicus_api.api.issuers_api.IssuersApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_clients_delete(self):
        """Test case for clients_delete

        Delete a client  # noqa: E501
        """
        pass

    def test_clients_get(self):
        """Test case for clients_get

        Get client  # noqa: E501
        """
        pass

    def test_clients_post(self):
        """Test case for clients_post

        Create a client  # noqa: E501
        """
        pass

    def test_clients_put(self):
        """Test case for clients_put

        Update a client  # noqa: E501
        """
        pass

    def test_clients_query(self):
        """Test case for clients_query

        Query Clients  # noqa: E501
        """
        pass

    def test_issuers_delete(self):
        """Test case for issuers_delete

        Delete an Issuer  # noqa: E501
        """
        pass

    def test_issuers_get(self):
        """Test case for issuers_get

        Get issuer  # noqa: E501
        """
        pass

    def test_issuers_post(self):
        """Test case for issuers_post

        Create an issuer  # noqa: E501
        """
        pass

    def test_issuers_put(self):
        """Test case for issuers_put

        Update an issuer  # noqa: E501
        """
        pass

    def test_issuers_query(self):
        """Test case for issuers_query

        Query Issuers  # noqa: E501
        """
        pass

    def test_redirects_delete(self):
        """Test case for redirects_delete

        Delete a client redirect  # noqa: E501
        """
        pass

    def test_redirects_get(self):
        """Test case for redirects_get

        Get redirect  # noqa: E501
        """
        pass

    def test_redirects_post(self):
        """Test case for redirects_post

        Create a redirect  # noqa: E501
        """
        pass

    def test_redirects_query(self):
        """Test case for redirects_query

        Query Redirects  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()
